From: "Rafael J. Wysocki" <rafael@kernel.org>
To: Linux ACPI <linux-acpi@vger.kernel.org>
Cc: LKML <linux-kernel@vger.kernel.org>,
"Dumbre, Saket" <saket.dumbre@intel.com>
Subject: [PATCH v1 11/26] ACPICA: Abort AML bytecode execution when executing AML_FATAL_OP
Date: Wed, 14 Jan 2026 13:25:33 +0100 [thread overview]
Message-ID: <3325491.5fSG56mABF@rafael.j.wysocki> (raw)
In-Reply-To: <12822121.O9o76ZdvQC@rafael.j.wysocki>
From: Armin Wolf <W_Armin@gmx.de>
The ACPI specification states that when executing AML_FATAL_OP,
the OS should log the fatal error event and shutdown in a timely
fashion.
Windows complies with this requirement by immediatly entering a
Bso_d, effectively aborting the execution of the AML bytecode in
question.
ACPICA however might continue with the AML bytecode execution
should acpi_os_signal() simply return AE_OK. This will cause issues
because ACPI BIOS implementations might assume that the Fatal()
operator does not return.
Fix this by aborting the AML bytecode execution in such a case
by returning AE_ERROR. Also turn struct acpi_signal_fatal_info into a
local variable because of its small size (12 bytes) and to ensure
that acpi_os_signal() always receives valid information about the
fatal ACPI BIOS error.
Link: https://github.com/acpica/acpica/commit/d516c7758ba6
Signed-off-by: Armin Wolf <W_Armin@gmx.de>
Signed-off-by: Rafael J. Wysocki <rafael.j.wysocki@intel.com>
---
drivers/acpi/acpica/exoparg3.c | 46 +++++++++++++---------------------
1 file changed, 18 insertions(+), 28 deletions(-)
diff --git a/drivers/acpi/acpica/exoparg3.c b/drivers/acpi/acpica/exoparg3.c
index bf08110ed6d2..c8c8c4e49563 100644
--- a/drivers/acpi/acpica/exoparg3.c
+++ b/drivers/acpi/acpica/exoparg3.c
@@ -10,6 +10,7 @@
#include <acpi/acpi.h>
#include "accommon.h"
#include "acinterp.h"
+#include <acpi/acoutput.h>
#include "acparser.h"
#include "amlcode.h"
@@ -51,8 +52,7 @@ ACPI_MODULE_NAME("exoparg3")
acpi_status acpi_ex_opcode_3A_0T_0R(struct acpi_walk_state *walk_state)
{
union acpi_operand_object **operand = &walk_state->operands[0];
- struct acpi_signal_fatal_info *fatal;
- acpi_status status = AE_OK;
+ struct acpi_signal_fatal_info fatal;
ACPI_FUNCTION_TRACE_STR(ex_opcode_3A_0T_0R,
acpi_ps_get_opcode_name(walk_state->opcode));
@@ -60,28 +60,23 @@ acpi_status acpi_ex_opcode_3A_0T_0R(struct acpi_walk_state *walk_state)
switch (walk_state->opcode) {
case AML_FATAL_OP: /* Fatal (fatal_type fatal_code fatal_arg) */
- ACPI_DEBUG_PRINT((ACPI_DB_INFO,
- "FatalOp: Type %X Code %X Arg %X "
- "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<\n",
- (u32)operand[0]->integer.value,
- (u32)operand[1]->integer.value,
- (u32)operand[2]->integer.value));
-
- fatal = ACPI_ALLOCATE(sizeof(struct acpi_signal_fatal_info));
- if (fatal) {
- fatal->type = (u32) operand[0]->integer.value;
- fatal->code = (u32) operand[1]->integer.value;
- fatal->argument = (u32) operand[2]->integer.value;
- }
+ fatal.type = (u32)operand[0]->integer.value;
+ fatal.code = (u32)operand[1]->integer.value;
+ fatal.argument = (u32)operand[2]->integer.value;
- /* Always signal the OS! */
+ ACPI_BIOS_ERROR((AE_INFO,
+ "Fatal ACPI BIOS error (Type 0x%X Code 0x%X Arg 0x%X)\n",
+ fatal.type, fatal.code, fatal.argument));
- status = acpi_os_signal(ACPI_SIGNAL_FATAL, fatal);
+ /* Always signal the OS! */
- /* Might return while OS is shutting down, just continue */
+ acpi_os_signal(ACPI_SIGNAL_FATAL, &fatal);
- ACPI_FREE(fatal);
- goto cleanup;
+ /*
+ * Might return while OS is shutting down, so abort the AML execution
+ * by returning an error.
+ */
+ return_ACPI_STATUS(AE_ERROR);
case AML_EXTERNAL_OP:
/*
@@ -93,21 +88,16 @@ acpi_status acpi_ex_opcode_3A_0T_0R(struct acpi_walk_state *walk_state)
* wrong if an external opcode ever gets here.
*/
ACPI_ERROR((AE_INFO, "Executed External Op"));
- status = AE_OK;
- goto cleanup;
+
+ return_ACPI_STATUS(AE_OK);
default:
ACPI_ERROR((AE_INFO, "Unknown AML opcode 0x%X",
walk_state->opcode));
- status = AE_AML_BAD_OPCODE;
- goto cleanup;
+ return_ACPI_STATUS(AE_AML_BAD_OPCODE);
}
-
-cleanup:
-
- return_ACPI_STATUS(status);
}
/*******************************************************************************
--
2.51.0
next prev parent reply other threads:[~2026-01-14 12:42 UTC|newest]
Thread overview: 30+ messages / expand[flat|nested] mbox.gz Atom feed top
2026-01-14 12:14 [PATCH v1 00/26] ACPICA: ACPICA 20251212 Rafael J. Wysocki
2026-01-14 12:16 ` [PATCH v1 01/26] ACPICA: Add support for the new ACPI Table: DTPR Rafael J. Wysocki
2026-01-14 12:17 ` [PATCH v1 02/26] ACPICA: ACPICA: replace ACPI_FREE() with acpi_ut_delete_object_desc() Rafael J. Wysocki
2026-01-14 12:18 ` [PATCH v1 03/26] ACPICA: Add UUID for Microsoft fan extensions Rafael J. Wysocki
2026-01-14 12:19 ` [PATCH v1 04/26] ACPICA: Add UUIDs associated with TPM 2.0 devices Rafael J. Wysocki
2026-01-14 12:20 ` [PATCH v1 05/26] ACPICA: Fix NULL pointer dereference in acpi_ev_address_space_dispatch() Rafael J. Wysocki
2026-03-16 17:46 ` Guenter Roeck
2026-03-17 20:44 ` Rafael J. Wysocki
[not found] ` <SJ0PR11MB500584283698C48BD431BE5D804BA@SJ0PR11MB5005.namprd11.prod.outlook.com>
2026-03-24 4:21 ` Guenter Roeck
2026-01-14 12:21 ` [PATCH v1 06/26] ACPICA: Add KEYP table definition Rafael J. Wysocki
2026-01-14 12:21 ` [PATCH v1 07/26] ACPICA: Add support for the Microsoft display mux _OSI string Rafael J. Wysocki
2026-01-14 12:22 ` [PATCH v1 08/26] ACPICA: iASL: Add definitions for the IOVT table Rafael J. Wysocki
2026-01-14 12:23 ` [PATCH v1 09/26] ACPICA: Add DTPR table support for the ASL compiler Rafael J. Wysocki
2026-01-14 12:24 ` [PATCH v1 10/26] ACPICA: Define DTPR structure related info tables and data template Rafael J. Wysocki
2026-01-14 12:25 ` Rafael J. Wysocki [this message]
2026-01-14 12:26 ` [PATCH v1 12/26] ACPICA: ACPI 6.4: PPTT: include all fields in subtable type1 Rafael J. Wysocki
2026-01-14 12:27 ` [PATCH v1 13/26] ACPICA: Fix asltests using the Fatal() opcode Rafael J. Wysocki
2026-01-14 12:29 ` [PATCH v1 14/26] ACPICA: Add GICv5 MADT structures Rafael J. Wysocki
2026-01-14 12:29 ` [PATCH v1 15/26] ACPICA: Add Arm IORT IWB node definitions Rafael J. Wysocki
2026-01-14 12:31 ` [PATCH v1 16/26] ACPICA: actbl2.h: ACPI 6.6: RAS2: Update Parameter Block structure Rafael J. Wysocki
2026-01-14 12:31 ` [PATCH v1 17/26] ACPICA: actbl3.h: ACPI 6.6: SRAT: New flag in Memory Affinity Structure Rafael J. Wysocki
2026-01-14 12:32 ` [PATCH v1 18/26] ACPICA: ACPI 6.6: Add _VDM (Voltage Domain) object Rafael J. Wysocki
2026-01-14 12:33 ` [PATCH v1 19/26] ACPICA: Create auxiliary ACPI_TPR_AUX_SR structure for iASL compiler Rafael J. Wysocki
2026-01-14 12:34 ` [PATCH v1 20/26] ACPICA: Fix Segmentation Fault error related to DTPR Rafael J. Wysocki
2026-01-14 12:35 ` [PATCH v1 21/26] ACPICA: Verify DTPR and TPR Instance buffer pointers Rafael J. Wysocki
2026-01-14 12:36 ` [PATCH v1 22/26] ACPICA: Cleanup comments and DTPR Table handle functions Rafael J. Wysocki
2026-01-14 12:37 ` [PATCH v1 23/26] ACPICA: Align comments in TPRn-related structures Rafael J. Wysocki
2026-01-14 12:38 ` [PATCH v1 24/26] ACPICA: Logfile: Changes for version 20251212 Rafael J. Wysocki
2026-01-14 12:39 ` [PATCH v1 25/26] ACPICA: Replace TPRn Base and Limit registers Rafael J. Wysocki
2026-01-14 12:40 ` [PATCH v1 26/26] ACPICA: Refactor for TPR Base/Limit registers bitmasks Rafael J. Wysocki
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=3325491.5fSG56mABF@rafael.j.wysocki \
--to=rafael@kernel.org \
--cc=linux-acpi@vger.kernel.org \
--cc=linux-kernel@vger.kernel.org \
--cc=saket.dumbre@intel.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox